enum TreeChange {
New,
Moved,
PendingMove,
Removed,
}Expand description
Tracks changes to a node’s relation to the tree within an update.
This is used to remove nodes from the accessibility tree’s cache when they are no longer in the tree.
Variants
New
The node was newly created in this update.
Moved
The node has been re-parented in this update.
PendingMove
The node has been added to its new parent, but not yet removed from its old parent.
When a node is moved within the tree, it must be both removed from its old parent and added to its new parent within the same update. This may happen in either order, depending on the relative positions of the node before and after it moves.
- If a node’s new parent is updated before its old parent, the node will be in a
TreeChange::PendingMovestate until its old parent is updated. We expect that it must later be removed from its old parent, at which point its state will be updated toTreeChange::Moved. - If a node’s old parent is updated before its new parent, the node will be first
TreeChange::Removedand thenTreeChange::Moved.
At the end of the update, we assert that there are no pending moves remaining.
Removed
The node is no longer a child of its previous parent.