pub struct AccessibilityTree {
nodes: FxHashMap<NodeId, ArcRefCell<AccessibilityNode>>,
opaque_node_to_id: FxHashMap<OpaqueNode, NodeId>,
tree_id: TreeId,
epoch: Epoch,
}Expand description
A retained, internal representation of the accessibility tree for a document.
accesskit only provides interchange types for tree updates and action requests, so we need
to define our own representation for incremental tree building.
Fields
nodes: FxHashMap<NodeId, ArcRefCell<AccessibilityNode>>opaque_node_to_id: FxHashMap<OpaqueNode, NodeId>tree_id: TreeIdepoch: EpochImplementations
Sourceimpl AccessibilityTree
impl AccessibilityTree
pub(crate) fn new(tree_id: TreeId, epoch: Epoch) -> Self
Sourcepub(crate) fn update_tree(
&mut self,
root_dom_node: &ServoLayoutNode<'_>,
) -> Option<TreeUpdate>
pub(crate) fn update_tree( &mut self, root_dom_node: &ServoLayoutNode<'_>, ) -> Option<TreeUpdate>
Update this tree based on the current state of the given DOM tree, and if anything changed,
return an accesskit::TreeUpdate representing what changed.
Sourcefn update_node_and_descendants(
&mut self,
dom_node: &ServoLayoutNode<'_>,
tree_update: &mut AccessibilityUpdate,
) -> bool
fn update_node_and_descendants( &mut self, dom_node: &ServoLayoutNode<'_>, tree_update: &mut AccessibilityUpdate, ) -> bool
Update this tree starting at the given DOM node, adding any changed nodes to the given
AccessibilityUpdate.
fn get_or_create_node( &mut self, dom_node: &ServoLayoutNode<'_>, ) -> ArcRefCell<AccessibilityNode>
fn assert_node_by_dom_node( &mut self, dom_node: &ServoLayoutNode<'_>, ) -> ArcRefCell<AccessibilityNode>
fn assert_node_by_id(&self, id: NodeId) -> ArcRefCell<AccessibilityNode>
fn id_for_opaque(&mut self, opaque: OpaqueNode) -> NodeId
pub(crate) fn epoch(&self) -> Epoch