struct AccessibilityUpdate {
changed_nodes: FxHashSet<NodeId>,
tree_changes: FxHashMap<NodeId, TreeChange>,
rooted_nodes: Option<FxHashSet<OpaqueNode>>,
}Expand description
Changes which have occurred during the current update.
Fields
changed_nodes: FxHashSet<NodeId>Nodes whose internal data has changed within the current update.
tree_changes: FxHashMap<NodeId, TreeChange>Nodes that changed their relation to the tree within the current update.
rooted_nodes: Option<FxHashSet<OpaqueNode>>Nodes which were removed from the DOM tree since the last reflow, which were rooted in
[AccessibilityData]. Only set if [pref::expensive_accessibility_test_assertions_enabled]
is set.
Implementations
Sourceimpl AccessibilityUpdate
impl AccessibilityUpdate
fn new(rooted_nodes: Option<FxHashSet<OpaqueNode>>) -> Self
fn add(&mut self, node: &mut AccessibilityNode)
fn set_tree_state_change(&mut self, node_id: NodeId, change: TreeChange)
Sourcefn finalize(self, tree: &mut AccessibilityTree) -> Option<TreeUpdate>
fn finalize(self, tree: &mut AccessibilityTree) -> Option<TreeUpdate>
Consume this AccessibilityUpdate, producing an accesskit::TreeUpdate if there have
been any changes to tree.
This will pass self into AccessibilityTree::remove_stale_nodes() to consume
Self::tree_changes.