struct AccessibilityUpdate {
changed_nodes: FxHashSet<NodeId>,
tree_changes: FxHashMap<NodeId, TreeChange>,
}Expand description
Changes which have occurred during the current update.
Fields
changed_nodes: FxHashSet<NodeId>Nodes whose internal data has changed within the current update.
tree_changes: FxHashMap<NodeId, TreeChange>Nodes that changed their relation to the tree within the current update.
Implementations
Sourceimpl AccessibilityUpdate
impl AccessibilityUpdate
fn new() -> Self
fn add(&mut self, node: &mut AccessibilityNode)
fn finalize(self, tree: &mut AccessibilityTree) -> Option<TreeUpdate>